# Generated by roxygen2: do not edit by hand

S3method(as.matrix,textmodel_doc2vec)
S3method(as.matrix,textmodel_docvector)
S3method(as.matrix,textmodel_word2vec)
S3method(as.matrix,textmodel_wordvector)
S3method(as.textmodel_doc2vec,dfm)
S3method(print,textmodel_doc2vec)
S3method(print,textmodel_docvector)
S3method(print,textmodel_word2vec)
S3method(print,textmodel_wordvector)
S3method(textmodel_doc2vec,tokens)
S3method(textmodel_lsa,dfm)
S3method(textmodel_lsa,tokens)
S3method(textmodel_word2vec,tokens)
export(analogy)
export(as.textmodel_doc2vec)
export(perplexity)
export(probability)
export(similarity)
export(textmodel_doc2vec)
export(textmodel_lsa)
export(textmodel_word2vec)
import(quanteda)
importFrom(methods,as)
importFrom(quanteda,dfm_weight)
importFrom(utils,head)
importFrom(utils,tail)
useDynLib(wordvector)
