#' Yahoo News summaries from 2014
#'
#' A corpus object containing 2,000 news summaries collected from Yahoo News via
#' RSS feeds in 2014. The title and description of the summaries are concatenated.
#' @name data_corpus_news2014
#' @references Watanabe, K. (2018). Newsmap: A semi-supervised approach to
#' geographical news classification. Digital Journalism, 6(3), 294–309.
#' https://doi.org/10.1080/21670811.2017.1293487
#' @source <https://www.yahoo.com/news/>
"data_corpus_news2014"
