# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

cpp_get_max_thread <- function() {
    .Call('_wordvector_cpp_get_max_thread', PACKAGE = 'wordvector')
}

cpp_word2vec <- function(xptr, model, size = 100L, window = 5L, sample = 0.001, withHS = FALSE, negative = 5L, threads = 1L, iterations = 5L, alpha = 0.05, type = 1L, doc2vec = FALSE, verbose = FALSE, normalize = TRUE) {
    .Call('_wordvector_cpp_word2vec', PACKAGE = 'wordvector', xptr, model, size, window, sample, withHS, negative, threads, iterations, alpha, type, doc2vec, verbose, normalize)
}

