% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{data_corpus_news2014}
\alias{data_corpus_news2014}
\title{Yahoo News summaries from 2014}
\format{
An object of class \code{corpus} (inherits from \code{character}) of length 20000.
}
\source{
\url{https://www.yahoo.com/news/}
}
\usage{
data_corpus_news2014
}
\description{
A corpus object containing 2,000 news summaries collected from Yahoo News via
RSS feeds in 2014. The title and description of the summaries are concatenated.
}
\references{
Watanabe, K. (2018). Newsmap: A semi-supervised approach to
geographical news classification. Digital Journalism, 6(3), 294–309.
https://doi.org/10.1080/21670811.2017.1293487
}
\keyword{datasets}
